# from magine.networks.databases import load_kegg, load_biogrid_network, \
#     load_signor, load_reactome_fi, load_hmdb_network
# from magine.networks.databases.biogrid_interactions import BioGridDownload
#
# fresh_download = True
# verbose = True
#
#
# def test_download():
#     bgn = BioGridDownload()
#     bgn.parse_network()
#
#
# def test_kegg():
#     kegg_network = load_kegg(fresh_download=fresh_download,
#                                     verbose=verbose)
#
#
# def test_hmbd():
#     hmdb_network = load_hmdb_network(fresh_download=fresh_download,
#                                      verbose=verbose)
#
#
# def test_biogrid():
#     biogrid_network = load_biogrid_network(fresh_download=fresh_download,
#                                            verbose=verbose)
#
#
# def test_signor():
#     signor_network = load_signor(fresh_download=fresh_download,
#                                  verbose=verbose)
#
#
# def test_reactome():
#     reactome_network = load_reactome_fi(verbose=verbose)
