<!doctype html>
<html>
<head>
  <title>Network | Basic usage</title>
</head>
<body>
<script type="text/javascript">
function setUpFrame() {
    var frame = window.frames["{{name|safe}}"];
    frame.runVis();
}
</script>
<iframe name="{{ name|safe }}" src="{{ filename|safe }}" width="100%;"
        height="800px"></iframe>
</body>
</html>